package com.boman.file.controller;

import com.boman.common.core.utils.SecurityUtils;
import com.boman.common.core.utils.obj.ObjectUtils;
import com.boman.common.redis.RedisKey;
import com.boman.common.redis.service.RedisService;
import com.boman.domain.dto.AjaxResult;
import com.boman.domain.dto.ExportExcelDto;
import com.boman.file.service.ISysFileService;
import com.boman.file.service.LocalSysFileServiceImpl;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

/**
 * @author shiqian
 * @date 2021年06月07日 16:24
 **/
@RestController
@RequestMapping("/common/file")
public class ExcelController {

    @Resource
    private ISysFileService fileService;
    @Resource
    private RedisService redisService;

    /**
     * 功能描述: 通用的导入接口
     *
     * @param multipartFile MultipartFile
     * @param tableName     tableName
     * @return void
     */
    @PostMapping("/import")
    public AjaxResult importExcelCommon(MultipartFile multipartFile, String tableName) {
        try {
            return fileService.importExcelCommon(multipartFile, tableName);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.error("失败");
    }


    /**
     * 功能描述: 通用的导出接口
     *
     * @param response response
     * @param dto      tableName
     *                 empty(true=>只带表头的excel，不含数据, false=>带数据的excel)
     *                 condition 查询条件
     * @return com.boman.domain.dto.AjaxResult
     */
    @PostMapping("/export")
    public AjaxResult exportExcelCommon(HttpServletResponse response, @RequestBody ExportExcelDto dto) {
        try {
            return fileService.exportExcelCommon(response, dto);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.error("失败");
    }

    /**
     * 功能描述: 通用的导出接口   异步
     *
     * @param response response
     * @param dto      tableName
     *                 empty(true=>只带表头的excel，不含数据, false=>带数据的excel)
     *                 condition 查询条件
     * @return com.boman.domain.dto.AjaxResult
     */
    @PostMapping("async/export")
    public AjaxResult asyncExportExcelCommon(HttpServletResponse response, @RequestBody ExportExcelDto dto) {
        try {
            return fileService.asyncExportExcelCommon(response, dto);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.error("失败");
    }

    /**
     * 功能描述: 是否可以下载
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping("/before/download")
    public AjaxResult canDownloadIfOrNot() {
        String redisKey = RedisKey.ASYNC_DOWNLOAD_SIGN + SecurityUtils.getUsername();
        String sign = redisService.getCacheObject(redisKey);
        if (LocalSysFileServiceImpl.DOWNLOADING.equals(sign)) {
            return AjaxResult.success(false);
        }

        return AjaxResult.error(true);
    }

    /**
     * 功能描述: 导出数据，只是sql不一样，其余的都是通用接口的内容
     *
     * @param response response
     * @param dto      tableName
     *                 empty(true=>只带表头的excel，不含数据, false=>带数据的excel)
     *                 condition 查询条件
     * @return com.boman.domain.dto.AjaxResult
     */
    @PostMapping("/export/statisticsByMonth")
    public AjaxResult statisticsByMonth(HttpServletResponse response, @RequestBody ExportExcelDto dto) {
        try {
            return fileService.statisticsByMonth(response, dto);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.error("失败");
    }



}
